@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
+ * Copyright (C) 2010-2011  Université de Bordeaux 1
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,7 +19,13 @@
 
 #include <starpu_task.h>
 
-struct starpu_task_list {
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
+struct starpu_task_list
+{
 	struct starpu_task *head;
 	struct starpu_task *tail;
 };
@@ -28,7 +34,7 @@ struct starpu_task_list {
 void starpu_task_list_init(struct starpu_task_list *list);
 
 /* Push a task at the front of a list */
-void starpu_task_list_push_front(struct starpu_task_list *list,	struct starpu_task *task);
+void starpu_task_list_push_front(struct starpu_task_list *list, struct starpu_task *task);
 
 /* Push a task at the back of a list */
 void starpu_task_list_push_back(struct starpu_task_list *list, struct starpu_task *task);
@@ -50,5 +56,18 @@ struct starpu_task *starpu_task_list_pop_front(struct starpu_task_list *list);
 
 /* Remove the element at the back of the list */
 struct starpu_task *starpu_task_list_pop_back(struct starpu_task_list *list);
-						
+
+/* Get the first task of the list */
+struct starpu_task *starpu_task_list_begin(struct starpu_task_list *list);
+
+/* Get the end of the list */
+struct starpu_task *starpu_task_list_end(struct starpu_task_list *list);
+
+/* Get the next task of the list. This is not erase-safe. */
+struct starpu_task *starpu_task_list_next(struct starpu_task *task);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* __STARPU_TASK_LIST_H__ */
