@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011  Université de Bordeaux 1
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2012  Université de Bordeaux 1
+ * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,15 +20,16 @@
 
 #include <sys/time.h>
 #include <starpu.h>
-#include <starpu_profiling.h>
 #include <core/jobs.h>
-#include <profiling/profiling.h>
 #include <common/utils.h>
 
-void _starpu_driver_update_job_feedback(starpu_job_t j, struct starpu_worker_s *worker_args,
-		struct starpu_task_profiling_info *profiling_info,
-		enum starpu_perf_archtype perf_arch,
-		struct timespec *codelet_start, struct timespec *codelet_end);
+void _starpu_driver_start_job(struct _starpu_worker *args, struct _starpu_job *j,
+			      struct timespec *codelet_start, int rank, int profiling);
+void _starpu_driver_end_job(struct _starpu_worker *args, struct _starpu_job *j, enum starpu_perf_archtype perf_arch,
+			    struct timespec *codelet_end, int rank, int profiling);
+void _starpu_driver_update_job_feedback(struct _starpu_job *j, struct _starpu_worker *worker_args,
+					enum starpu_perf_archtype perf_arch,
+					struct timespec *codelet_start, struct timespec *codelet_end, int profiling);
 
 void _starpu_block_worker(int workerid, pthread_cond_t *cond, pthread_mutex_t *mutex);
 
