@@ -22,7 +22,11 @@
 
 #include "../helper.h"
 
+#ifdef STARPU_QUICK_CHECK
+static unsigned ntasks = 40;
+#else
 static unsigned ntasks = 40000;
+#endif
 
 #ifdef STARPU_USE_CUDA
 extern void increment_cuda(void *descr[], __attribute__ ((unused)) void *_args);
@@ -95,10 +99,6 @@ int main(int argc, char **argv)
 
         FPRINTF(stderr, "Token: %u\n", token);
 
-#ifdef STARPU_SLOW_MACHINE
-	ntasks /= 10;
-#endif
-
 	for(i=0; i<ntasks; i++)
 	{
                 ret = starpu_data_acquire_cb(token_handle, STARPU_W, callback, NULL);  // recv
